﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;
using System.Windows.Forms;

namespace C1.Chart.Test
{
    public class ChartTestOptions
    {
        public ChartTestOptions()
        {
            NPoints = "100,1000";
            NRuns = 100;
            Controls = "C1Chart,FlexChart,FlexChartDX";
            Width = 640;
            Height = 480;
        }

        [System.ComponentModel.Browsable(false)]
        public bool SaveImage { get; set; }
        public string NPoints { get; set; }
        public int NRuns { get; set; }

        [System.ComponentModel.Browsable(false)]
        public string Controls { get; set; }

        public int Width { get; set; }
        public int Height { get; set; }
    }

    public class ChartTests
    {
        static int seed = 12345;
        static Random rnd = new Random(seed);

        private ChartTestOptions options = new ChartTestOptions();
        private bool isWinForms;

        public ChartTests(bool isWinForms = false)
        {
            this.isWinForms = isWinForms;
        }

        public ChartTestOptions Options
        {
            get { return options; }
        }

        public TestResult Run(IChart chart, PlotType type, int npts, int n, Func<string,bool> runStart, Func<IChart,bool> endTest = null)
        {
            chart.PlotType = type;
            var info = chart.GetInfo();

            var tr = new TestRunner(isWinForms);
            var pts = CreateData(npts);
            var res = tr.Run( () =>
            {
                chart.Clear();
                chart.Create(pts);
                endTest?.Invoke(chart);
            }, (i) =>
            {
                return runStart(string.Format("{0}-{1} {2}/{3}", info, type.ToString(), i+1, n));
            }, n);

            res.Info = info;
            res.NPts = npts;
            return res;
        }

        public TestResultGroup[] RunGroup(Func<string, IChart> action, Func<string, bool> runStart=null, Func<IChart,bool> endTest = null, PlotType[] types = null)
        {
            var nptss = ParseNPts();
            var ctrls = Options.Controls.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);

            var list = new List<TestResultGroup>();
            foreach (var npts in nptss)
                list.AddRange(RunGroup(action, runStart, endTest, ctrls, Options.NRuns, npts, types));

            return list.ToArray();
        }

        TestResultGroup[] RunGroup(Func<string,IChart> action, Func<string,bool> runStart, Func<IChart,bool> endTest, string[] parameters, int n, int npts, PlotType[] types = null)
        {
            if (types == null)
                types = (PlotType[])Enum.GetValues(typeof(PlotType));

            var tgs = new TestResultGroup[parameters.Length];
            for (var i = 0; i < parameters.Length; i++)
            {
                var tg = new TestResultGroup();
                tg.Info = "Total";
                tg.NPts = npts;
                var chart = action(parameters[i]);
                if (chart == null)
                    continue; // fail ?

                //tg.GroupTitle = string.Format("{0} {1}pts {2}runs {3}x{4}", chart.GetInfo(), npts, n, Options.Width, Options.Height);
                tg.GroupTitle = string.Format("{0}", chart.GetInfo());

                for (var j = 0; j < types.Length; j++)
                {
                    var tr = Run(chart, types[j], npts, n, runStart, endTest);
                    
                    tr.Info = types[j].ToString();
                    tg.Results.Add(tr);

                    if (Options.SaveImage)
                    {
                        var ctrl = (Control)chart.GetControl();
                        using (var bmp = new Bitmap(ctrl.Width, ctrl.Height))
                        {
                            ctrl.DrawToBitmap(bmp, new Rectangle(0, 0, ctrl.Width, ctrl.Height));
                            var fn = string.Format("{0}-{1}.png", tg.GroupTitle, tr.Info);
                            bmp.Save(fn, System.Drawing.Imaging.ImageFormat.Png);
                        }
                    }
                }

                tgs[i] = tg;
            }

            return tgs;
        }

        Dictionary<int, List<DataPoint>> dict = new Dictionary<int, List<DataPoint>>();

        public List<DataPoint> CreateData(int npts)
        {
            if (dict.ContainsKey(npts))
                return dict[npts];

            var list = new List<DataPoint>();
            for (var i = 0; i < npts; i++)
                list.Add(new DataPoint(i, rnd.NextDouble()));
            dict[npts] = list;
            return list;
        }

        int[] ParseNPts()
        {
            var npts = new List<int>();
            var ss = Options.NPoints.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            foreach (var s in ss)
            {
                int i = 0;
                if (int.TryParse(s, out i) && i > 0)
                    npts.Add(i);
            }

            if (npts.Count == 0)
                npts.Add(100);

            return npts.ToArray();
        }
    }
}
