﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace C1.Chart.Test
{
    class TestRunner
    {
        bool waitForFinalizers;

        public TestRunner(bool waitForFinalizers = false)
        {
            this.waitForFinalizers = waitForFinalizers;
        }

        double RunAction(Action action)
        {
            var sw = new Stopwatch();
            sw.Start();
            try
            {
                action();
            }
            catch
            {
                return double.NaN;
            }
            sw.Stop();
            return sw.Elapsed.TotalMilliseconds;
        }

        public TestResult Run(Action action, Func<int,bool> start, int n, int nwarm = 1)
        {
            // warm up
            for (var i = 0; i < nwarm; i++)
                RunAction(action);

            var res = new TestResult();

            var sum = 0.0;
            var times = new List<double>();
            for (var i = 0; i < n; i++)
            {
                if (start != null)
                    start(i);
                ForceGcCollect();

                var t = RunAction(action);
                sum += t;
                times.Add(t);

                ForceGcCollect();
            }

            res.AddData(times);
            return res;
        }

        private void ForceGcCollect()
        {
            GC.Collect();

            if (waitForFinalizers)
            {
                GC.WaitForPendingFinalizers();
                GC.Collect();
            }
        }
    }

    public class TestResult
    {
        protected List<double> _times;

        public string Title { get; set; }
        public string Info { get; set; }
        public int NPts { get; set; }

        public double AverageTime { get; set; }
        public double MinTime { get; set; }
        public double MaxTime { get; set; }

        public double StDeviation { get; set; }

        public List<double> Times { get { return _times; } }

        public void AddData(List<double> times)
        {
            _times = times;

            AverageTime = times.Average();
            MinTime = times.Min();
            MaxTime = times.Max();

            var s = 0.0;
            for (var i = 0; i < times.Count; i++)
            {
                var d = times[i] - AverageTime;
                s += d * d;
            }
            StDeviation = Math.Sqrt(s / (times.Count - 1));
        }

        public virtual string Format()
        {
            return string.Format("{0,-15}{1,-15} {2,6:n2} +/- {3,6:p1}\r\n",
                Title, Info, AverageTime, StDeviation / AverageTime);
        }

        public virtual string FormatCsv()
        {
            return string.Format("{0},{1},{2},{3:n1}\r\n",
                Info, NPts, AverageTime, StDeviation / AverageTime * 100);
        }
    }

    public class TestResultGroup : TestResult
    {
        List<TestResult> _results = new List<TestResult>();
        List<TestResultGroup> _groups = new List<TestResultGroup>();

        public string GroupTitle { get; set; }

        public List<TestResult> Results
        {
            get { return _results; }
        }
        //public List<TestResultGroup> Groups
        //{
        //    get { return _groups; }
        //}

        public override string Format()
        {
            if (Results.Count == 0)
                return "";

            _times = new List<double>();

            var cnt = Results[0].Times.Count;
            for (var i = 0; i < cnt; i++)
            {
                var sum = 0.0;
                for (var j = 0; j < Results.Count; j++)
                    sum += Results[j].Times[i];
                _times.Add(sum);
            }
            AddData(_times);

            var sb = new StringBuilder();
            //sb.AppendFormat("{0}\r\n{1}", Title, Info);
            if (!string.IsNullOrEmpty(GroupTitle))
                sb.AppendFormat("{0} {1}pts\r\n", GroupTitle, NPts);
            sb.Append(base.Format());
            for (var i = 0; i < Results.Count; i++)
                sb.Append(Results[i].Format());
            sb.Append("\r\n");
            return sb.ToString();
        }

        public override string FormatCsv()
        {
            if (Results.Count == 0)
                return "";

            _times = new List<double>();

            var cnt = Results[0].Times.Count;
            for (var i = 0; i < cnt; i++)
            {
                var sum = 0.0;
                for (var j = 0; j < Results.Count; j++)
                    sum += Results[j].Times[i];
                _times.Add(sum);
            }
            AddData(_times);

            var sb = new StringBuilder();
            
            if (!string.IsNullOrEmpty(GroupTitle))
            {
                sb.Append(GroupTitle);
                sb.Append(",");
            }
            sb.Append(base.FormatCsv());
            for (var i = 0; i < Results.Count; i++)
            {
                if (!string.IsNullOrEmpty(GroupTitle))
                {
                    sb.Append(GroupTitle);
                    sb.Append(",");
                }
                sb.Append(Results[i].FormatCsv());
            }
            return sb.ToString();
        }

    }
}
